from typing import TypeVar

from .ast_utils import get_extended_name, get_variable_name, set_extended_name

T = TypeVar('T')


def Named(item: T) -> T:
    name = get_variable_name(with_raise=False)
    # if the same type has already been defined, then we get the previous object
    if get_extended_name(item) is not None:
        return item
    set_extended_name(item, name)
    return item



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union

if TYPE_CHECKING:
    from .petit_ts import TSTypeStore


class BaseHandler(ABC):

    @staticmethod
    @abstractmethod
    def should_handle(cls: Any, store: TSTypeStore,
                      origin: Optional[type], args: List[Any]) -> bool: ...

    @staticmethod
    @abstractmethod
    def build(cls: Any, store: TSTypeStore, origin: Optional[type],
              args: List[Any]) -> Tuple[Optional[str], Union[str, Dict[str, Any]]]: ...


class BasicHandler(BaseHandler):
    @staticmethod
    @abstractmethod
    def build(cls: Any, store: TSTypeStore, origin: Optional[type],
              args: List[Any]) -> Tuple[Optional[str], str]: ...


class ClassHandler(BaseHandler):
    @staticmethod
    @abstractmethod
    def is_mapping() -> bool: ...

    @staticmethod
    def is_inline(cls: type) -> bool:
        return False

    @staticmethod
    @abstractmethod
    def build(cls, store: TSTypeStore, origin: Optional[type],
              args: List[Any]) -> Tuple[Optional[str], Union[str, Dict[str, Any]]]: ...



from typing import Any, List, Union

INLINE_TOKEN = "__inline__"

NoneType = type(None)

DEFAULT_TYPES = {
    str(bool): "boolean",
    str(None): "void",
    str(Any): "any",
    str(int): "number /*int*/",
    str(float): "number /*float*/",
    str(str): "string",
    str(dict): "object",
    str(list): "any[]",
    str(List[Any]): "any[]",
    str(List): "any[]",
    str(List[int]): "number[]",
    str(List[str]): "string[]",
}

pseudo_classes = Union[dict, None, Any]



import inspect
import threading
from typing import (Any, Set, Tuple, TypeVar, Union, get_args, get_origin,
                    get_type_hints)

from .const import NoneType, pseudo_classes


class SafeCounter:
    def __init__(self, value: int = 0):
        self.value = value
        self.lock = threading.Lock()

    def increment(self, value: int = 1) -> int:
        with self.lock:
            self.value += value
        return self.value

    def decrement(self, value: int = 1) -> int:
        with self.lock:
            self.value -= value
        return self.value


def is_optional(cls: pseudo_classes) -> Tuple[bool, Tuple[Any]]:
    args = get_args(cls)
    return get_origin(cls) == Union and NoneType in args, args


def is_array(origin, args) -> bool:
    return origin == list and len(args) == 1


def is_mapping(origin, args) -> bool:
    return origin == dict and len(args) == 2


def __get_generic_params(cls: type, l: set) -> None:
    if inspect.isclass(cls):
        for type_ in get_type_hints(cls).values():
            if isinstance(type_, TypeVar):
                l.add(type_)
                continue
            args = get_args(type_)
            if len(args) > 0:
                __get_generic_params(type_, l)
    else:
        for arg in get_args(cls):
            if isinstance(arg, TypeVar):
                l.add(arg)
            args = get_args(arg)
            if len(args) > 0:
                __get_generic_params(arg, l)


def get_generic_params(cls: type) -> Set[TypeVar]:
    l = set()
    __get_generic_params(cls, l)
    return l


def is_generic(cls: type) -> Tuple[bool, Set[TypeVar]]:
    """Tells if a given class is generic
    """
    fields = get_generic_params(cls)
    return len(fields) > 0, fields



from setuptools import setup

with open('readme.md', 'r') as f :
    readme_content = f.read()

setup(
    name='petit_ts',
    version='0.1.5',
    description='Convert your python types to typescript',
    packages=['petit_ts'],
    url='https://github.com/Plawn/petit_ts',
    license='apache-2.0',
    author='Plawn',
    author_email='plawn.yay@gmail.com',
    long_description=readme_content,
    long_description_content_type="text/markdown",
    python_requires='>=3.8',
    install_requires=['executing'],
)
